package cn.com.infosec.jce.oscca;

import cn.com.infosec.jce.provider.InfosecProvider;
import cn.com.infosec.math.ec.ECCurve;
import cn.com.infosec.math.ec.ECFieldElement;
import cn.com.infosec.math.ec.ECPoint;
import cn.com.infosec.util.BigIntegers;
import java.io.PrintStream;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.security.SecureRandom;
import java.security.Security;

public class SM2 {
   static final int SM2_SIZE = 32;
   static final BigInteger gmp;
   static final BigInteger gma;
   static final BigInteger gmb;
   static final ECCurve gmec256;
   static final BigInteger gmgx;
   static final BigInteger gmgy;
   static final ECPoint gmg;
   static final BigInteger gmn;
   public static final byte[] defaultIdAndLength;
   private static final byte[] sm2abgxgy;
   private static final SecureRandom random;

   static {
      BigInteger var0;
      BigInteger var10000 = var0 = new BigInteger;
      var10000.<init>("FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFF", 16);
      gmp = var10000;
      BigInteger var1;
      var10000 = var1 = new BigInteger;
      var10000.<init>("FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFC", 16);
      gma = var10000;
      BigInteger var2;
      var10000 = var2 = new BigInteger;
      var10000.<init>("28E9FA9E9D9F5E344D5A9E4BCF6509A7F39789F515AB8F92DDBCBD414D940E93", 16);
      gmb = var10000;
      ECCurve var3 = gmec256 = new ECCurve.Fp(var0, var1, var2);
      BigInteger var10001 = gmgx = new BigInteger("32C4AE2C1F1981195F9904466A39C9948FE30BBFF2660BE1715A4589334C74C7", 16);
      BigInteger var10002 = var0 = new BigInteger;
      var10002.<init>("BC3736A2F4F6779C59BDCEE36B692153D0A9877CC62A474002DF32E52139F0A0", 16);
      gmgy = var10002;
      gmg = var3.createPoint(var10001, var0, false);
      gmn = new BigInteger("FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123", 16);
      defaultIdAndLength = new byte[]{0, -128, 49, 50, 51, 52, 53, 54, 55, 56, 49, 50, 51, 52, 53, 54, 55, 56};
      sm2abgxgy = new byte[]{-1, -1, -1, -2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 0, 0, 0, 0, -1, -1, -1, -1, -1, -1, -1, -4, 40, -23, -6, -98, -99, -97, 94, 52, 77, 90, -98, 75, -49, 101, 9, -89, -13, -105, -119, -11, 21, -85, -113, -110, -35, -68, -67, 65, 77, -108, 14, -109, 50, -60, -82, 44, 31, 25, -127, 25, 95, -103, 4, 70, 106, 57, -55, -108, -113, -29, 11, -65, -14, 102, 11, -31, 113, 90, 69, -119, 51, 76, 116, -57, -68, 55, 54, -94, -12, -10, 119, -100, 89, -67, -50, -29, 107, 105, 33, 83, -48, -87, -121, 124, -58, 42, 71, 64, 2, -33, 50, -27, 33, 57, -16, -96};
      random = getRandom();
   }

   private static SecureRandom getRandom() {
      SecureRandom var10000;
      SecureRandom var0;
      try {
         var10000 = SecureRandom.getInstance("SHA1PRNG");
      } catch (Exception var1) {
         var0 = new SecureRandom.<init>();
         return var0;
      }

      var0 = var10000;
      return var0;
   }

   public static SM2 getInstance() {
      return SM2.SM2Holder.instance;
   }

   private static ECPoint kmg(ECPoint p, BigInteger k) {
      BigInteger var10001 = k;
      ECPoint var10003 = p;
      ECPoint var10004 = p;
      ECCurve p1 = gmec256;
      ECFieldElement k1 = var10004.getX();
      ECFieldElement var2 = var10003.getY();
      return (new JPECPoint.Fp(p1, k1, var2)).multiply(var10001).toECPoint();
   }

   public static byte[] signHash(byte[] msg_hash, byte[] privkey) {
      if (msg_hash.length == 32 && privkey.length == 32) {
         BigInteger var2;
         var2 = new BigInteger.<init>(1, msg_hash);

         BigInteger var3;
         BigInteger var4;
         BigInteger var5;
         BigInteger var6;
         BigInteger msg_hash2;
         do {
            int msg_hash1 = gmn.bitLength();

            BigInteger var10000;
            do {
               var10000 = var3 = new BigInteger;
               var10000.<init>(msg_hash1, random);
            } while(var10000.equals(var4 = BigInteger.ZERO) || var3.compareTo(var5 = gmn) >= 0 || (var6 = kmg(gmg, var3).getX().toBigInteger().mod(var5)).equals(var4) || var2.add(var6).compareTo(var5) == 0);

            msg_hash2 = new BigInteger.<init>(1, privkey);
            var6 = var2.add(var6).mod(var5);
         } while((msg_hash2 = BigInteger.ONE.add(msg_hash2).modInverse(var5).multiply(var3.add(msg_hash2.multiply(var6).negate())).mod(var5)).equals(var4));

         return xy2sign(BigIntegers.asUnsignedByteArray(var6), BigIntegers.asUnsignedByteArray(msg_hash2));
      } else {
         return null;
      }
   }

   private static ECPoint implShamirsTrick(JPECPoint P, BigInteger k, JPECPoint Q, BigInteger l) {
      int var10000 = Math.max(k.bitLength(), l.bitLength());
      JPECPoint var4 = P.add(Q);
      JPECPoint var5 = P.getInfinity();

      for(int var6 = var10000 - 1; var6 >= 0; --var6) {
         var5 = var5.twice();
         if (k.testBit(var6)) {
            if (l.testBit(var6)) {
               var5 = var5.add(var4);
            } else {
               var5 = var5.add(P);
            }
         } else if (l.testBit(var6)) {
            var5 = var5.add(Q);
         }
      }

      return var5.toECPoint();
   }

   public static boolean verifyHash(byte[] msg_hash, byte[] signature, byte[] pubkey) {
      if (msg_hash.length != 32) {
         return false;
      } else if (signature.length != 64 && signature.length != 65 && signature.length < 69) {
         return false;
      } else if (pubkey.length != 64 && pubkey.length != 65) {
         return false;
      } else {
         BigInteger var3;
         var3 = new BigInteger.<init>(1, msg_hash);
         BigInteger var4;
         BigInteger msg_hash1;
         BigInteger signature1;
         if (signature.length >= 69) {
            byte[] var10000 = signature;
            msg_hash = new byte[32];
            signature = new byte[32];
            sign2xy(var10000, msg_hash, signature);
            var4 = new BigInteger.<init>(1, msg_hash);
            msg_hash1 = new BigInteger.<init>(1, signature);
            signature1 = msg_hash1;
         } else {
            msg_hash = new byte[32];
            byte var10 = 0;
            if (signature[0] == 4 && signature.length == 65) {
               var10 = 1;
            }

            byte var10001 = var10;
            System.arraycopy(signature, var10, msg_hash, 0, 32);
            var4 = new BigInteger.<init>(1, msg_hash);
            System.arraycopy(signature, var10001 + 32, msg_hash, 0, 32);
            signature1 = new BigInteger.<init>(1, msg_hash);
         }

         BigInteger var5;
         if (var4.compareTo(msg_hash1 = BigInteger.ONE) >= 0 && var4.compareTo(var5 = gmn) < 0) {
            if (signature1.compareTo(msg_hash1) >= 0 && signature1.compareTo(var5) < 0) {
               msg_hash1 = var4.add(signature1).mod(var5);
               ECPoint pubkey1 = octect2point(pubkey);
               JPECPoint var11 = SM2Constants.jg;
               BigInteger var12 = signature1;
               JPECPoint.Fp signature2;
               signature2 = new JPECPoint.Fp.<init>(pubkey1);
               return implShamirsTrick(var11, var12, signature2, msg_hash1).getX().toBigInteger().add(var3).mod(var5).compareTo(var4) == 0;
            } else {
               return false;
            }
         } else {
            return false;
         }
      }
   }

   public static ECPoint octect2point(byte[] pb) {
      byte[] var1 = new byte[32];
      byte[] var2 = new byte[32];
      int var3;
      if ((var3 = pb.length) != 64) {
         if (var3 != 65) {
            return null;
         }

         byte var6 = 1;
         byte var4 = 33;
         if (pb[0] != 4) {
            return null;
         }

         System.arraycopy(pb, var6, var1, 0, 32);
         System.arraycopy(pb, var4, var2, 0, 32);
      } else {
         System.arraycopy(pb, 0, var1, 0, 32);
         System.arraycopy(pb, 32, var2, 0, 32);
      }

      ECCurve var10000 = gmec256;
      BigInteger var10001 = new BigInteger(1, var1);
      BigInteger pb1;
      pb1 = new BigInteger.<init>(1, var2);
      return var10000.createPoint(var10001, pb1, false);
   }

   public static ECPoint octect2point(byte[] pb, int offset, int length) {
      byte[] var3 = new byte[32];
      byte[] var4 = new byte[32];
      if (length != 64) {
         if (length != 65) {
            return null;
         }

         length = offset + 1;
         offset += 33;
         if (pb[0] != 4) {
            return null;
         }

         System.arraycopy(pb, length, var3, 0, 32);
         System.arraycopy(pb, offset, var4, 0, 32);
      } else {
         byte[] var10000 = pb;
         byte[] var10001 = pb;
         int pb1 = offset + 0;
         offset += 32;
         System.arraycopy(var10001, pb1, var3, 0, 32);
         System.arraycopy(var10000, offset, var4, 0, 32);
      }

      ECCurve var7 = gmec256;
      BigInteger var8 = new BigInteger(1, var3);
      BigInteger pb2;
      pb2 = new BigInteger.<init>(1, var4);
      return var7.createPoint(var8, pb2, false);
   }

   public static void sign2xy(byte[] sign, byte[] x, byte[] y) {
      byte var3 = 0;
      byte var4 = 0;
      if (sign[3] == 33) {
         var3 = 1;
      }

      if (sign[var3 + 37] == 33) {
         var4 = 1;
      }

      for(int var5 = 0; var5 < 32; ++var5) {
         x[var5] = sign[var3 + 4 + var5];
         y[var5] = sign[var3 + 38 + var4 + var5];
      }

   }

   public static byte[] xy2sign(byte[] x, byte[] y) {
      int var2 = 0;
      byte var3 = 0;
      if (x.length == 32 && (x[0] & 255) >= 128) {
         var2 = 1;
      }

      if (y.length == 32 && (y[0] & 255) >= 128) {
         var3 = 1;
      }

      byte[] var4;
      byte[] var10001 = var4 = new byte[var2 + 70 + var3];
      var4[0] = 48;
      var4[1] = (byte)(var2 + 68 + var3);
      var4[2] = 2;
      var4[var2 + 3] = 0;
      var10001[3] = (byte)(var2 + 32);
      int var5 = 32 - x.length;

      int var6;
      for(var6 = 0; var6 < var5; ++var6) {
         var4[var2 + 4 + var6] = 0;
      }

      int var7;
      for(var6 = 0; var6 < x.length; ++var6) {
         var7 = var2 + 4 + var6 + var5;
         var4[var7] = x[var6];
      }

      var4[var2 + 36] = 2;
      var4[var7 = var2 + 38] = 0;
      int x1 = var2 + 37;
      var4[x1] = (byte)(var3 + 32);
      int var8 = 32 - y.length;

      for(x1 = 0; x1 < var8; ++x1) {
         var4[var7 + var3 + x1] = 0;
      }

      for(x1 = 0; x1 < y.length; ++x1) {
         var2 = var7 + var3 + x1 + var8;
         var4[var2] = y[x1];
      }

      return var4;
   }

   public static byte[] signData(byte[] id, byte[] data, byte[] privKey, byte[] pubkey) {
      SM3 var4;
      var4 = new SM3.<init>();
      if (id == null) {
         var4.update(defaultIdAndLength);
      } else {
         int var10003 = id.length;
         var4.update((byte)(id.length >> 8));
         var4.update((byte)(var10003 & 255));
         var4.update(id);
      }

      var4.update(sm2abgxgy);
      if (pubkey == null) {
         BigInteger id1;
         id1 = new BigInteger.<init>(1, privKey);
         ECPoint id2 = SM2Constants.jg.multiply(id1).toECPoint();
         byte[] var10002 = pubkey = new byte[32];
         byte[] var12 = BigIntegers.asUnsignedByteArray(id2.getX().toBigInteger());
         int var5 = 32 - var12.length;
         int var6 = var12.length;
         System.arraycopy(var12, 0, pubkey, var5, var6);
         var4.update(var10002);
         var10002 = BigIntegers.asUnsignedByteArray(id2.getY().toBigInteger());
         int id3 = 32 - var10002.length;
         var5 = var10002.length;
         System.arraycopy(var10002, 0, pubkey, id3, var5);
         var4.update(pubkey);
      } else {
         var4.update(ByteBuffer.allocate(64).put(pubkey, 1, 64).array());
      }

      var4.digest(id = new byte[32]);
      SM3 pubkey1;
      SM3 var10000 = pubkey1 = new SM3;
      pubkey1.<init>();
      pubkey1.update(id);
      var10000.update(data);
      byte[] var11 = id = new byte[32];
      pubkey1.digest(id);
      return signHash(var11, privKey);
   }

   public static byte[] signData(byte[] data, byte[] privKey) {
      return signData((byte[])null, data, privKey, (byte[])null);
   }

   public static byte[] signData(byte[] data, byte[] privKey, byte[] pubKey) {
      return signData((byte[])null, data, privKey, pubKey);
   }

   public static boolean verifyData(byte[] id, byte[] data, byte[] signature, byte[] pubkey) {
      if (data != null && signature != null && pubkey != null) {
         SM3 var4;
         var4 = new SM3.<init>();
         if (id == null) {
            var4.update(defaultIdAndLength);
         } else {
            int var10003 = id.length;
            var4.update((byte)(id.length >> 8));
            var4.update((byte)(var10003 & 255));
            var4.update(id);
         }

         var4.update(sm2abgxgy);
         var4.update(ByteBuffer.allocate(64).put(pubkey, 1, 64).array());
         var4.digest(id = new byte[32]);
         SM3 var10000 = var4 = new SM3;
         var4.<init>();
         var4.update(id);
         var10000.update(data);
         byte[] var5 = id = new byte[32];
         var4.digest(id);
         return verifyHash(var5, signature, pubkey);
      } else {
         return false;
      }
   }

   public static boolean verifyData(byte[] data, byte[] signature, byte[] pubkey) {
      return verifyData((byte[])null, data, signature, pubkey);
   }

   public static byte[] kdf(byte[] x2y2, int keySize) {
      if (keySize < 1) {
         return null;
      } else {
         SM3 var2;
         var2 = new SM3.<init>();
         byte var3 = 32;
         int var10001 = keySize;
         int var10002 = keySize;
         keySize /= var3;
         int var4 = var10002 % var3;
         byte[] var5 = new byte[var10001];
         byte[] var6 = new byte[32];
         int var7 = 1;

         byte[] var10007;
         for(int var8 = 0; var8 < keySize; ++var7) {
            boolean var14;
            byte[] var15;
            SM3 var10000;
            byte var10003;
            SM3 var10004;
            SM3 var10005;
            byte[] var10006;
            try {
               var10000 = var2;
               var15 = var5;
               var10002 = var8;
               var10003 = var3;
               var10004 = var2;
               var10005 = var2;
               var10006 = x2y2;
               var10007 = ByteBuffer.allocate(4).putInt(var7).array();
            } catch (Exception var12) {
               var14 = false;
               throw new RuntimeException("internal error");
            }

            byte[] var9 = var10007;

            try {
               var10005.update(var10006);
               var10004.update(var9);
            } catch (Exception var11) {
               var14 = false;
               throw new RuntimeException("internal error");
            }

            var10002 *= var10003;

            try {
               var10000.digest(var15, var10002);
            } catch (Exception var10) {
               var14 = false;
               throw new RuntimeException("internal error");
            }

            ++var8;
         }

         if (var4 != 0) {
            var10007 = x2y2;
            x2y2 = ByteBuffer.allocate(4).putInt(var7).array();
            var2.update(var10007);
            var2.update(x2y2);
            var2.digest(var6);
            int x2y21 = keySize * var3;
            System.arraycopy(var6, 0, var5, x2y21, var4);
         }

         return var5;
      }
   }

   public static byte[] encrypt(byte[] msg, byte[] pubkey) {
      byte[] var2 = new byte[msg.length + 97];
      int var3 = gmn.bitLength();

      SecureRandom var4;
      label30: {
         SecureRandom var10000;
         try {
            var10000 = SecureRandom.getInstance("SHA1PRNG");
         } catch (Exception var8) {
            var4 = new SecureRandom.<init>();
            break label30;
         }

         var4 = var10000;
      }

      BigInteger var5;
      ECPoint var6;
      ECPoint var7;
      ECPoint var11;
      do {
         while((var5 = (new BigInteger(var3, var4)).mod(gmn)).compareTo(BigInteger.ZERO) <= 0) {
         }

         var6 = SM2Constants.jg.multiply(var5).toECPoint();
         var7 = octect2point(pubkey);
      } while((var11 = (new JPECPoint.Fp(var7)).multiply(var5).toECPoint()).isInfinity());

      byte[] var10003 = BigIntegers.asUnsignedByteArray(var6.getX().toBigInteger());
      var2[0] = 4;
      int pubkey1 = 33 - var10003.length;
      var3 = var10003.length;
      System.arraycopy(var10003, 0, var2, pubkey1, var3);
      byte[] var10002 = BigIntegers.asUnsignedByteArray(var6.getY().toBigInteger());
      pubkey1 = 65 - var10002.length;
      var3 = var10002.length;
      System.arraycopy(var10002, 0, var2, pubkey1, var3);
      byte[] var12;
      (var12 = new byte[65])[0] = 4;
      byte[] var16 = BigIntegers.asUnsignedByteArray(var11.getX().toBigInteger());
      pubkey1 = 33 - var16.length;
      var3 = var16.length;
      System.arraycopy(var16, 0, var12, pubkey1, var3);
      byte[] var15 = BigIntegers.asUnsignedByteArray(var11.getY().toBigInteger());
      pubkey1 = 65 - var15.length;
      var3 = var15.length;
      System.arraycopy(var15, 0, var12, pubkey1, var3);
      byte[] var13 = kdf(ByteBuffer.allocate(64).put(var12, 1, 64).array(), msg.length);

      for(pubkey1 = 0; pubkey1 < msg.length; ++pubkey1) {
         var3 = pubkey1 + 97;
         var2[var3] = (byte)(msg[pubkey1] ^ var13[pubkey1]);
      }

      SM3 pubkey2;
      SM3 var17 = pubkey2 = new SM3;
      pubkey2.<init>();
      pubkey2.update(ByteBuffer.allocate(32).put(var12, 1, 32).array());
      pubkey2.update(msg);
      pubkey2.update(ByteBuffer.allocate(32).put(var12, 33, 32).array());
      var17.digest(var2, 65);
      return var2;
   }

   public static boolean decrypt(byte[] ct, byte[] privkey, byte[] pt) {
      if (ct.length >= 98 && privkey.length == 32) {
         int var3;
         byte[] var4 = ByteBuffer.allocate(var3 = ct.length - 97).put(ct, 97, var3).array();
         byte[] var5 = ByteBuffer.allocate(32).put(ct, 65, 32).array();
         ECPoint ct1;
         privkey = BigIntegers.asUnsignedByteArray((ct1 = octect2point(ByteBuffer.allocate(65).put(ct, 0, 65).array()).multiply(new BigInteger(1, privkey))).getX().toBigInteger());
         byte[] var6;
         byte[] var10000 = var6 = new byte[64];
         int ct2 = 32 - privkey.length;
         int privkey1 = privkey.length;
         System.arraycopy(privkey, 0, var6, ct2, privkey1);
         byte[] var12 = BigIntegers.asUnsignedByteArray(ct1.getY().toBigInteger());
         ct2 = 64 - var12.length;
         privkey1 = var12.length;
         System.arraycopy(var12, 0, var6, ct2, privkey1);
         ct = kdf(var10000, var3);

         for(privkey1 = 0; privkey1 < var3; ++privkey1) {
            pt[privkey1] = (byte)(var4[privkey1] ^ ct[privkey1]);
         }

         SM3 var11 = new SM3();
         var11.update(ByteBuffer.allocate(32).put(var6, 0, 32).array());
         var11.update(pt);
         var11.update(ByteBuffer.allocate(32).put(var6, 32, 32).array());
         byte[] var7;
         var11.digest(var7 = new byte[32]);

         for(ct2 = 0; ct2 < 32; ++ct2) {
            if ((var7[ct2] & 255) != (var5[ct2] & 255)) {
               return false;
            }
         }

         return true;
      } else {
         return false;
      }
   }

   public static void main(String[] args) {
      Exception var10000;
      label290: {
         boolean var10001;
         try {
            Security.addProvider(new InfosecProvider());
         } catch (Exception var49) {
            var10000 = var49;
            var10001 = false;
            break label290;
         }

         SM2 var59;
         byte[] var60;
         try {
            var59 = getInstance();
            var60 = new byte[32];
         } catch (Exception var48) {
            var10000 = var48;
            var10001 = false;
            break label290;
         }

         byte[] args1 = var60;

         byte[] var10002;
         try {
            var10002 = new byte[65];
         } catch (Exception var47) {
            var10000 = var47;
            var10001 = false;
            break label290;
         }

         byte[] var1 = var10002;

         byte[] var10003;
         try {
            var10003 = new byte[32];
         } catch (Exception var46) {
            var10000 = var46;
            var10001 = false;
            break label290;
         }

         byte[] var2 = var10003;
         String var3 = "This is string to sign";

         try {
            var10003 = new byte[72];
         } catch (Exception var45) {
            var10000 = var45;
            var10001 = false;
            break label290;
         }

         byte[] var4 = var10003;
         boolean var5 = false;
         byte var6 = 1;

         try {
            var59.genKeyPair(var60, var10002);
         } catch (Exception var44) {
            var10000 = var44;
            var10001 = false;
            break label290;
         }

         int var7 = 0;

         int var61;
         byte[] var62;
         while(var7 < 32) {
            int var65;
            try {
               var62 = var2;
               var65 = var7;
               var61 = args1[var7];
            } catch (Exception var43) {
               var10000 = var43;
               var10001 = false;
               break label290;
            }

            var10003 = var1;
            int var10004 = ++var7;

            byte var68;
            try {
               var68 = var10003[var10004];
            } catch (Exception var42) {
               var10000 = var42;
               var10001 = false;
               break label290;
            }

            var61 ^= var68;

            try {
               var62[var65] = (byte)var61;
            } catch (Exception var41) {
               var10000 = var41;
               var10001 = false;
               break label290;
            }
         }

         long var67;
         try {
            var67 = System.currentTimeMillis();
         } catch (Exception var40) {
            var10000 = var40;
            var10001 = false;
            break label290;
         }

         long var56 = var67;

         for(int var9 = 0; var9 < var6; ++var9) {
            SM3 var63;
            try {
               var62 = var2;
               var60 = args1;
               var63 = new SM3;
            } catch (Exception var39) {
               var10000 = var39;
               var10001 = false;
               break label290;
            }

            SM3 var54 = var63;

            try {
               var54.<init>();
               var54.update(var3.getBytes());
               var63.digest(var2);
               var62 = signHash(var62, var60);
            } catch (Exception var38) {
               var10000 = var38;
               var10001 = false;
               break label290;
            }

            var4 = var62;
         }

         try {
            var67 = System.currentTimeMillis();
         } catch (Exception var37) {
            var10000 = var37;
            var10001 = false;
            break label290;
         }

         long var57 = var67;

         PrintStream var73;
         StringBuffer var74;
         try {
            var73 = System.out;
            var74 = (new StringBuffer(String.valueOf(var6))).append(" sm2sign operation(s) has elapsed ");
         } catch (Exception var36) {
            var10000 = var36;
            var10001 = false;
            break label290;
         }

         long var64 = var56 = var57 - var56;

         try {
            var74 = var74.append(var64).append(" ms\n or ");
         } catch (Exception var35) {
            var10000 = var35;
            var10001 = false;
            break label290;
         }

         double var58;
         double var66 = (var58 = (double)var6 * 1000.0) / (double)var56;

         try {
            var73.println(var74.append(var66).append(" ops").toString());
            var67 = System.currentTimeMillis();
         } catch (Exception var34) {
            var10000 = var34;
            var10001 = false;
            break label290;
         }

         var56 = var67;

         boolean var75;
         for(int var11 = 0; var11 < var6; ++var11) {
            SM3 var69;
            try {
               var62 = var2;
               var60 = var4;
               var10002 = var1;
               var69 = new SM3;
            } catch (Exception var33) {
               var10000 = var33;
               var10001 = false;
               break label290;
            }

            SM3 var55 = var69;

            try {
               var55.<init>();
               var55.update(var3.getBytes());
               var69.digest(var2);
               var75 = verifyHash(var62, var60, var10002);
            } catch (Exception var32) {
               var10000 = var32;
               var10001 = false;
               break label290;
            }

            var5 = var75;
         }

         long var76;
         try {
            var75 = var5;
            var76 = System.currentTimeMillis();
         } catch (Exception var31) {
            var10000 = var31;
            var10001 = false;
            break label290;
         }

         long var51 = var76;

         StringBuffer var70;
         PrintStream var77;
         try {
            var77 = System.out;
            var70 = (new StringBuffer(String.valueOf(var6))).append(" sm2verify operation(s) has elapsed ");
         } catch (Exception var30) {
            var10000 = var30;
            var10001 = false;
            break label290;
         }

         long var71 = var51 -= var56;

         try {
            var70 = var70.append(var71).append(" ms\n or ");
         } catch (Exception var29) {
            var10000 = var29;
            var10001 = false;
            break label290;
         }

         double var72 = var58 / (double)var51;

         try {
            var77.println(var70.append(var72).append(" ops").toString());
         } catch (Exception var28) {
            var10000 = var28;
            var10001 = false;
            break label290;
         }

         if (!var75) {
            try {
               System.out.println("SM2 Verification failed!");
            } catch (Exception var27) {
               var10000 = var27;
               var10001 = false;
               break label290;
            }
         } else {
            try {
               System.out.println("SM2 verification is OK!");
            } catch (Exception var26) {
               var10000 = var26;
               var10001 = false;
               break label290;
            }
         }

         try {
            var75 = verifyHash(new byte[]{-5, 57, -93, -5, 77, -34, 90, -12, 3, 68, -71, -31, 126, -113, -56, 48, -123, 114, 14, -62, -57, 97, 87, 121, 81, 83, 34, -95, -72, -72, 30, 4}, new byte[]{48, 69, 2, 32, 125, -50, 77, 45, 116, -89, 49, 27, -119, -111, -15, 11, 19, -46, -69, 30, -19, -5, 23, -40, 64, -80, 47, -95, 120, 83, -48, -108, -116, -15, 35, 49, 2, 33, 0, -73, 84, -116, -31, -78, 67, 89, 2, 31, 4, 28, 121, 35, 25, 33, -75, -11, 9, -57, 91, -44, -95, -108, -112, -101, 92, 40, -89, -108, 43, -10, 35}, new byte[]{4, -23, -55, -53, 126, -119, -59, 55, -43, -56, 56, 78, 70, 25, -107, -34, -116, 34, 109, 42, 3, 100, 74, 91, -53, -65, 69, -98, -57, 9, -17, 27, -66, -98, -70, 83, 45, -73, -36, 66, -127, -74, -79, -6, 106, -29, -31, 112, -107, -75, 82, -102, -51, -69, 20, 80, 24, 99, -30, 15, 27, 124, -118, -109, -110});
         } catch (Exception var25) {
            var10000 = var25;
            var10001 = false;
            break label290;
         }

         if (!var75) {
            try {
               System.out.println("SM2 Vector Verification failed!");
            } catch (Exception var24) {
               var10000 = var24;
               var10001 = false;
               break label290;
            }
         } else {
            try {
               System.out.println("SM2 Vector verification is OK!");
            } catch (Exception var23) {
               var10000 = var23;
               var10001 = false;
               break label290;
            }
         }

         String var52 = "Beijing Infosec Technologies Co.,Ltd. is a Chinese company based in 北京.";
         String var78 = "pony wang";

         try {
            var60 = signData(var78.getBytes(), var52.getBytes(), args1, (byte[])null);
         } catch (Exception var22) {
            var10000 = var22;
            var10001 = false;
            break label290;
         }

         byte[] var53 = var60;

         try {
            var75 = verifyData(var78.getBytes(), var52.getBytes(), var53, var1);
         } catch (Exception var21) {
            var10000 = var21;
            var10001 = false;
            break label290;
         }

         if (!var75) {
            try {
               System.out.println("SM2 with ID Verification failed!");
            } catch (Exception var20) {
               var10000 = var20;
               var10001 = false;
               break label290;
            }
         } else {
            try {
               System.out.println("SM2 with ID verification is OK!");
            } catch (Exception var19) {
               var10000 = var19;
               var10001 = false;
               break label290;
            }
         }

         try {
            var62 = encrypt(var52.getBytes(), var1);
         } catch (Exception var18) {
            var10000 = var18;
            var10001 = false;
            break label290;
         }

         var60 = var62;

         try {
            var10002 = var60;
            var60 = args1;
            var61 = var10002.length;
         } catch (Exception var17) {
            var10000 = var17;
            var10001 = false;
            break label290;
         }

         var61 -= 97;

         try {
            var10002 = new byte[var61];
         } catch (Exception var16) {
            var10000 = var16;
            var10001 = false;
            break label290;
         }

         args1 = var10002;

         try {
            var75 = decrypt(var62, var60, var10002);
         } catch (Exception var15) {
            var10000 = var15;
            var10001 = false;
            break label290;
         }

         if (!var75) {
            try {
               System.out.println("SM2 encryption/decryption failed!");
            } catch (Exception var14) {
               var10000 = var14;
               var10001 = false;
               break label290;
            }
         } else {
            try {
               System.out.println("SM2 encryption/decryption is OK!\n" + new String(args1));
            } catch (Exception var13) {
               var10000 = var13;
               var10001 = false;
               break label290;
            }
         }

         try {
            System.out.println("End!");
            return;
         } catch (Exception var12) {
            var10000 = var12;
            var10001 = false;
         }
      }

      var10000.printStackTrace();
   }

   public void genKeyPair(byte[] privKey, byte[] pubKeyX, byte[] pubKeyY) {
      int var7 = gmn.bitLength();

      BigInteger var4;
      ECPoint var5;
      do {
         var4 = (new BigInteger(var7, random)).mod(gmn);
         ECPoint var10000 = gmg;
         var5 = var10000.multiply(var4);
         ECPoint var6 = kmg(var10000, var4);
         System.out.println("KG:\n" + var5.toString() + "\n" + var6.toString());
      } while(var5.isInfinity());

      for(var7 = 0; var7 < 32; ++var7) {
         privKey[var7] = 0;
         pubKeyX[var7] = 0;
         pubKeyY[var7] = 0;
      }

      byte[] var10002 = BigIntegers.asUnsignedByteArray(var4);
      var7 = 32 - var10002.length;
      int var9 = var10002.length;
      System.arraycopy(var10002, 0, privKey, var7, var9);
      byte[] var10001 = BigIntegers.asUnsignedByteArray(var5.getX().toBigInteger());
      var7 = 32 - var10001.length;
      int privKey1 = var10001.length;
      System.arraycopy(var10001, 0, pubKeyX, var7, privKey1);
      byte[] var10 = BigIntegers.asUnsignedByteArray(var5.getY().toBigInteger());
      var7 = 32 - var10.length;
      privKey1 = var10.length;
      System.arraycopy(var10, 0, pubKeyY, var7, privKey1);
   }

   public void genKeyPair(byte[] privkey, byte[] pubkey) {
      int var5 = gmn.bitLength();

      BigInteger var3;
      ECPoint var4;
      do {
         var3 = (new BigInteger(var5, random)).mod(gmn);
      } while((var4 = SM2Constants.jg.multiply(var3).toECPoint()).isInfinity());

      byte[] var7 = BigIntegers.asUnsignedByteArray(var3);

      for(var5 = 0; var5 < 32; ++var5) {
         privkey[var5] = 0;
      }

      var5 = 32 - var7.length;
      int var8 = var7.length;
      System.arraycopy(var7, 0, privkey, var5, var8);
      pubkey[0] = 4;

      for(var5 = 1; var5 < 65; ++var5) {
         pubkey[var5] = 0;
      }

      byte[] var10001 = BigIntegers.asUnsignedByteArray(var4.getX().toBigInteger());
      var5 = 33 - var10001.length;
      int privkey1 = var10001.length;
      System.arraycopy(var10001, 0, pubkey, var5, privkey1);
      byte[] var10000 = BigIntegers.asUnsignedByteArray(var4.getY().toBigInteger());
      var5 = 65 - var10000.length;
      privkey1 = var10000.length;
      System.arraycopy(var10000, 0, pubkey, var5, privkey1);
   }

   public boolean verifyHash(byte[] msg_hash, byte[] r, byte[] s, byte[] x, byte[] y) {
      byte[] var10000 = msg_hash;
      byte[] var6 = new byte[64];
      System.arraycopy(r, 0, var6, 0, 32);
      System.arraycopy(s, 0, var6, 32, 32);
      (msg_hash = new byte[65])[0] = 4;
      System.arraycopy(x, 0, msg_hash, 1, 32);
      System.arraycopy(y, 0, msg_hash, 33, 32);
      return verifyHash(var10000, var6, msg_hash);
   }

   static class SM2Holder {
      static SM2 instance = new SM2();
   }
}
